@@ -17,12 +17,16 @@ package SDCommon;
 use base qw/ Exporter /;
 our @EXPORT = qw(writeCfg withecho withechoNoPrompt needs_upsTagUrl
 needs_upsCurrentUrl needs_tagsUrl url insvn long_path oldSvnDirsCheck
-load_dirs set_statusref svnMkdirP start_ssh);
+load_dirs set_statusref svnMkdirP start_ssh _g);
 #### common ####
-our @EXPORT_OK = qw ($tagVersion $upVersion $package $version $opt_verbose $opt_noninteractive %c $opt_nosave $opt_quiet $epoch $version);
+our @EXPORT_OK = qw ($tagVersion $upVersion $package $version
+$opt_verbose $opt_noninteractive %c $opt_nosave $opt_quiet $epoch
+$version);
 use vars qw($package $epoch $version $opt_verbose);
 use Cwd;
 use SVN::Client;
+use POSIX qw(locale_h);
+use Locale::gettext;
 use strict;
 
 #my $SDCommon::opt_quiet;
@@ -94,13 +98,11 @@ sub withecho {
    if(system(@_)) {
       my $cdir = Cwd::getcwd ;
       $cdir = "<unknown>" unless ($cdir);
-      print STDERR "Command $cmd failed in ".
-         $cdir.
-         ", how to continue now? [Qri?]: ";
+      printf STDERR (_g("Command '%s' failed in '%s', how to continue now? [Qri?]: "), $cmd, $cdir);
       prompt:
       my $ans = <STDIN>;
       if($ans =~ /^q$/i or $ans =~ /^$/) {
-         exwerror("Aborting.\n");
+         exwerror(_g("Aborting.\n"));
       }
       elsif($ans =~ /^i$/i) {
          return 1;
@@ -109,8 +111,9 @@ sub withecho {
          goto retry;
       }
       else {
-         print STDERR "Invalid selection! " if $ans !~ /^\?/;
-         print STDERR "The choices are: Quit (q), Retry (r), Ignore & continue (i).\n";
+         print STDERR _g("Invalid selection! ") if $ans !~ /^\?/;
+         # Translators: retain the letters q, r and i as untranslated.
+         print STDERR _g("The choices are: Quit (q), Retry (r), Ignore & continue (i).\n");
          print STDERR "[Qri?]: ";
          goto prompt;
       }
@@ -158,7 +161,8 @@ sub svnMkdirP {
    return 0 if ($missingurl =~ /^$/ ) ;
    chomp ( $svntempdir=`mktemp -d` ) ;
    system ( "mkdir -p $svntempdir/$missingurl" ) ;
-   withecho("svn", "import", "$svntempdir", "$tempurl", "-m", "$scriptname Creating $missingurl directory." );
+   my $msg = sprintf (_g("Creating %s directory."), $missingurl);
+   withecho("svn", "import", "$svntempdir", "$tempurl", "-m", "$scriptname $msg" );
    system "rm -fr $svntempdir 2>/dev/null" ;
 }
 
@@ -167,14 +171,14 @@ sub oldSvnDirsCheck {
    my $dir = $_[0];
    @svndirs=`find $dir -name .svn`;
    if(@svndirs) {
-      print "Found conflicting .svn directories in the upstream source:\n";
+      print _g("Found conflicting .svn directories in the upstream source:\n");
       if($SDCommon::opt_verbose) {
          print @svndirs;
       }
       else {
-         print "use -v to display the files\n";
+         print _g("use -v to display the files\n");
       }
-      print "Hint: use the uclean program to fix upstream source tarball\n";
+      print _g("Hint: use the uclean program to fix upstream source tarball\n");
       exit 1;
    }
 }
@@ -218,9 +222,9 @@ sub insvn {
    $url=$proto.$url;
    return undef if !defined $testurl;
    return  $inSvn{$testurl} if $inSvn{$testurl};
-   print "Repository lookup, probing $url...\n";
+   printf (_g("Repository lookup, probing '%s' ...\n"), $url);
    open(INFO, "env LC_ALL=C svn ls $url 2>/dev/null |") or
-      die ("Can't open svn ls $url: $!");
+      die (sprintf (_g("Can't open svn ls '%s': %s"), $url, $!));
    @junk=<INFO>;
    if (close INFO) {
       $inSvn{$testurl}=$url;
@@ -258,7 +262,7 @@ sub search {
    # see 433532 for more details
    $basedir=Cwd::getcwd if(!$basedir);
    if($basedir=~/:\/\//) { # huch, URL was specified as $basedir?
-      print "W: $var not specified anywhere, looking in the local repository...\n";
+      printf (_g("W: %s not specified anywhere, looking in the local repository...\n"), $var);
       @testsvn=("$basedir/../$pre/$suf",
       "$basedir/../../$pre/$package/$suf",
       "$basedir/../../$pre/$suf/$package");
@@ -271,7 +275,7 @@ sub search {
    }
    else {
       if($SDCommon::opt_verbose) {
-         print "I: Trying blind lookup for resource directories in SVN repository.\n";
+         print _g("I: Trying blind lookup for resource directories in SVN repository.\n");
          print "D: $pre, $suf, $var, $basedir.\n";
       }
       @testloc=(long_path("$basedir/../$pre/$suf"),
@@ -281,14 +285,14 @@ sub search {
       if ($dir && -d $dir) { $SDCommon::c{$var}=$dir; return;}
    }
    for (@testsvn) {
-     print "Looking in SVN for: @testsvn\n";
+     printf (_g("Looking in SVN for: %s\n", $_));
       if ($_ && insvn($_)) { 
-         print "I: adding the URLs to the $cfgFile to skip the check later.\n" if(!$SDCommon::opt_nosave);
+         printf (_g("I: adding the URLs to the '%s' to skip the check later.\n"), $cfgFile) if(!$SDCommon::opt_nosave);
          $SDCommon::c{$var}=insvn($_);
          return;
       }
       else {
-         print "Failed, assuming non-existent directory...";
+         print _g("Failed, assuming non-existent directory...");
       }
    }
 }
@@ -296,31 +300,35 @@ sub search {
 # if still undefined, try blind search in the repository
 sub needs_upsTagUrl {
    search("branches","upstream", "upsTagUrl", url("."));
-   exwerror "upsTagUrl setting is required, but could not be found in $cfgFile or repository tree." if (!defined $SDCommon::c{"upsTagUrl"});
+   exwerror sprintf(_g("upsTagUrl setting is required, but could not be found in %s or repository tree."), $cfgFile) if (!defined $SDCommon::c{"upsTagUrl"});
 }
 sub needs_upsCurrentUrl {
    search("branches","upstream/current", "upsCurrentUrl", url("."));
    if( !defined $SDCommon::c{"upsCurrentUrl"} ) { # maybe layout 2?
         search("branches/upstream","current", "upsCurrentUrl", url("."));
    }
-   exwerror "upsCurrentUrl setting is required, but could not be found in $cfgFile or repository tree." if (!defined $SDCommon::c{"upsCurrentUrl"});
+    # Translators: %s is the name of the config file.
+   exwerror sprintf (_g("upsCurrentUrl setting is required, but could not be found in %s or repository tree."), $cfgFile) if (!defined $SDCommon::c{"upsCurrentUrl"});
 }
 sub needs_tagsUrl {
     my $hereurl=url(".");
     search("tags", "", "tagsUrl", $hereurl);
     if( !defined $SDCommon::c{"tagsUrl"} && $hereurl=~/(.*)\/branches\// ) { # oh, crap...
-        print "Branch directory detected in URL, testing more possible locations\n";
+        print _g("Branch directory detected in URL, testing more possible locations\n");
         search("tags", "", "tagsUrl", "$1/$package");
     }
-    exwerror "tagsUrl setting is required, but could not be found in $cfgFile or repository tree." if (!defined $SDCommon::c{"tagsUrl"});
+    # Translators: %s is the name of the config file.
+    exwerror sprintf (_g("tagsUrl setting is required, but could not be found in %s or repository tree."), $cfgFile) if (!defined $SDCommon::c{"tagsUrl"});
 }
 sub writeCfg {
    my $cfgFile = shift;
-   print "I: Configuration saving was overrided ($cfgFile should be intact).\n" if ($SDCommon::opt_nosave==1 and (defined $SDCommon::opt_verbose) );
+    # Translators: %s is the name of the config file.
+   printf (_g("I: Configuration saving was overrided (%s should be intact).\n"), $cfgFile) if ($SDCommon::opt_nosave==1 and (defined $SDCommon::opt_verbose) );
    return 1 if $SDCommon::opt_nosave==1;
    if($_[0]){$cfgFile = $_[0];}
-   print "I: Trying to save the configuration to $cfgFile.\n" if defined $SDCommon::opt_verbose;
-   open(CFG, ">$cfgFile") || die "Could not open $cfgFile for writing.\n";
+    # Translators: %s is the name of the config file.
+   printf (_g("I: Trying to save the configuration to %s.\n"), $cfgFile) if defined $SDCommon::opt_verbose;
+   open(CFG, ">$cfgFile") || die sprintf(_g("Could not open %s for writing.\n"), $cfgFile);
    foreach (sort(keys %SDCommon::c)) {
       print CFG "$_=".$SDCommon::c{$_}."\n" if defined $SDCommon::c{$_};
    }
@@ -331,7 +339,7 @@ sub writeCfg {
 sub sd_exit {
    my $ret = shift||0;
    if($SDCommon::opt_nosave==0) {
-      print "Writing config: $cfgFile\n\n\n" if $SDCommon::opt_verbose;
+      printf (_g("Writing config: %s"), $cfgFile."\n\n\n") if $SDCommon::opt_verbose;
       &writeCfg(".svn/deb-layout");
    }
    # print STDERR "Return-Code: $ret";
@@ -343,7 +351,7 @@ sub init {
 
    open(INFOC, "env LC_ALL=C svn info debian/changelog |");  @junk=<INFOC>;
    if(!close(INFOC)) {
-      exwerror "Not started from the Trunk directory or not a valid SVN repository. Aborting.\n";
+      exwerror _g("Not started from the Trunk directory or not a valid SVN repository. Aborting.\n");
    }
 
 #   `head -n1 debian/changelog` =~ /^(\S+)\s*\((.+)\)/;
@@ -375,18 +383,19 @@ sub init {
          $SDCommon::upVersion=~s/^.*://;
          $SDCommon::upVersion=~s/(.*)-([^-]+)/$1/;
       }
-      print STDERR "I: Got package name and version from
-      debian/changelog.\n" if $SDCommon::opt_verbose;
+      print STDERR _g("I: Got package name and version from debian/changelog.\n")
+      	if $SDCommon::opt_verbose;
    }
    else { 
-      exwerror "E: Not started from the trunk/PACKAGE directory (debian/changelog garbled?).\n";
+      exwerror _g("E: Not started from the trunk/PACKAGE directory (debian/changelog garbled?).\n");
    }
-   print "
-   Package name: $SDCommon::package
-   Current upstream version: $SDCommon::upVersion
-   Debian tag: $SDCommon::tagVersion
+   printf (_g("
+   Package name: %s
+   Current upstream version: %s
+   Debian tag: %s
 
-   " if $SDCommon::opt_verbose;
+   "), $SDCommon::package, $SDCommon::upVersion, $SDCommon::tagVersion)
+     if $SDCommon::opt_verbose;
 
    $SDCommon::opt_nosave=1;
 
@@ -426,7 +435,7 @@ sub readLayoutProps {
       # make sure we don't override anything from the command line
       # or some other prioritary levels
       $SDCommon::c{$prop}=$val if (!defined $SDCommon::c{$prop});
-      print "Importing layout information via Subversion properties... \n" if ( $nonempty == 0 );
+      print _g("Importing layout information via Subversion properties... \n") if ( $nonempty == 0 );
       print "\t$prop: $val\n";
       $nonempty = 1 ;
       }
@@ -436,22 +445,23 @@ sub readLayoutProps {
 sub printLayout {
    our ($msg) = @_;
    my $nonempty = 0 ;
-   $msg = "Layout info" if ( !defined $msg ) ;
+   $msg = _g("Layout info") if ( !defined $msg ) ;
    print "$msg:\n" ;
    foreach (sort(keys %SDCommon::c)) {
       print "\t$_=".$SDCommon::c{$_}."\n" if (defined $SDCommon::c{$_}) ; #if($_=~/^\// && ! -e $_);
       $nonempty = 1;
       };
-   print "\t(no information)\n" if ( $nonempty == 0 ) ;
+   printf ("\t(%s)\n", _g("no information")) if ( $nonempty == 0 ) ;
 }
 
 sub readLayoutFromFile {
    our ($layoutFile, $formalFileName) = @_;
 
    if(! -e $layoutFile) {
-      print "No layout information imported through $formalFileName.\n$layoutFile does not exist.\n" if ( defined $opt_verbose ) ;
+      printf (_g("No layout information imported through %s.\n%s does not exist.\n"),
+        $formalFileName, $layoutFile) if ( defined $opt_verbose ) ;
    } else {
-      print "Importing layout information from $layoutFile.\n" if ( defined $opt_verbose ) ;
+      printf (_g("Importing layout information from %s.\n"), $layoutFile) if ( defined $opt_verbose ) ;
       if(-r $layoutFile) {
          open(CFG, "<$layoutFile") or die ("Can't open $layoutFile: $!");
          while(<CFG>) {
@@ -483,16 +493,16 @@ sub configure {
    #  props, .svn/deb-layout, command line
 
    &readLayoutProps;
-   &printLayout ("Versioned layout properties") if (defined $SDCommon::opt_verbose);
+   &printLayout (_g("Versioned layout properties")) if (defined $SDCommon::opt_verbose);
 
-   &readLayoutFromFile ( $defCfgFile, "the versioned file layout" ) ;
+   &readLayoutFromFile ( $defCfgFile, _g("the versioned file layout") ) ;
    # status of the versioned layout info
-   &printLayout ("Versioned layout information") if (defined $SDCommon::opt_verbose);
+   &printLayout (_g("Versioned layout information")) if (defined $SDCommon::opt_verbose);
 
-   &readLayoutFromFile ( $cfgFile, "the local override" ) ;
+   &readLayoutFromFile ( $cfgFile, _g("the local override") ) ;
 
    # give a nice status of the currently set options
-   &printLayout ("Layout information with local overrides") if (defined $SDCommon::opt_verbose);
+   &printLayout (_g("Layout information with local overrides")) if (defined $SDCommon::opt_verbose);
 
    # always redetect them; keep them in the same config base just for the
    # record
@@ -501,17 +511,17 @@ sub configure {
       $SDCommon::c{"trunkDir"}=getcwd;
    }
    else {
-      exwerror "We are not in a working copy of SVN trunk directory";
+      exwerror _g("We are not in a working copy of SVN trunk directory");
    }
 
    foreach(values(%SDCommon::c)) {
-      exwerror "\nThe directory $_ does not exist!
+      exwerror sprintf (_g("\nThe directory %s does not exist!
 
 Create this directory or fix the setting in .svn/deb-layout or remove that
 line and let svn-buildpackage redetect the value. Also check the associated
 URL.
 
-" if($_=~/^\// && ! -e $_);
+"), $_) if($_=~/^\// && ! -e $_);
    }
    # lookup for local locations and get URLs from them if needed
    search("build-area", "", "buildArea");
@@ -528,15 +538,15 @@ URL.
   @opt_override = split(/,|\ |\r|\n/,join(',',@opt_override));
   for(@opt_override) {
     if(/(.*)=(.*)/) {
-      print "Overriding variable: $1 with $2\n" if (defined $SDCommon::opt_verbose);
+      printf _g("Overriding variable: %s with %s\n"), $1, $2 if (defined $SDCommon::opt_verbose);
       $SDCommon::c{$1}=$2;
     }
     else {
-      print "Warning, unable to parse the override string: $_\n";
+      printf (_g("Warning, unable to parse the override string: %s\n"), $_);
     }
   }
 
-   &printLayout ("Complete layout information");
+   &printLayout (_g("Complete layout information"));
 
    &writeCfg(".svn/deb-layout");
 
@@ -570,8 +580,8 @@ sub check_uncommited {
        }
    }
    if (@conflicts) {
-      print "E: Found unresolved issues: \n\n@conflicts\n";
-      exwerror "E: Resolve them manually before continuing\n";
+      printf (_g("E: Found unresolved issues: %s"), "\n\n@conflicts\n");
+      exwerror sprintf (_g("E: Resolve them manually before continuing\n"));
    }
    close(SVN);
 }
@@ -614,7 +624,8 @@ sub load_dirs {
     if( ! withechoNoPrompt("svn", "co", $url, $tmpdir) ) {
         # svn sucks here, import does create subdirs, but mkdir has no -p switch
         withecho "mkdir", "-p", $tmpdir;
-        withecho("svn", "-m", "Creating trunk directory", "import", $tmpdir, $url);
+        my $msg = _g("Creating trunk directory");
+        withecho("svn", "-m", "$msg", "import", $tmpdir, $url);
         unlink $tmpdir;
         withecho("svn", "co", $url, $tmpdir);
     }
@@ -651,22 +662,23 @@ sub load_dirs {
             $ctx->add($_, 1);
         } elsif($tmp{$_} == 11) {
            if ($SDCommon::opt_ignored_files_action) {
-              exwerror "Action for ignored files must be either 'import' or 'skip'!\nAborting!"
+			   # Translators: 'import' and 'skip' are not to be translated
+              exwerror _g("Action for ignored files must be either 'import' or 'skip'!\nAborting!")
                 unless ($SDCommon::opt_ignored_files_action eq "import" or $SDCommon::opt_ignored_files_action eq "skip");
               $ctx->add($_, 1) if $SDCommon::opt_ignored_files_action eq "import";
            } else {
               unless ($SDCommon::opt_noninteractive) {
-                 print STDERR "One or more files are ignore due to (global) subversion ignore patterns,\n\thow to procede (quit, import, skip)? [Qis?]: ";
+                 printf STDERR _g("One or more files are ignore due to (global) subversion ignore patterns,\n\thow to procede (quit, import, skip)? [Qis?]: ");
                  prompt:
                  my $ans = <STDIN>;
-                 exwerror "Aborting!\n" if ($ans =~ /^$/ or $ans =~ /^q/i);
+                 exwerror sprintf (_g("Aborting!\n")) if ($ans =~ /^$/ or $ans =~ /^q/i);
                  if ($ans =~ /^i/i) {
                     $ctx->add($_, 1);
                     $SDCommon::opt_ignored_files_action = "import";
                  } elsif ($ans =~ /^s/i) {
                     $SDCommon::opt_ignored_files_action = "skip";
                  } else {
-                    print STDERR "Your choices are:\n\tQuit (q), import (i) which means to add ignored files or skip (s)\n\twhich means to NOT include those files.\n";
+                    printf STDERR _g("Your choices are:\n\tQuit (q), import (i) which means to add ignored files or skip (s)\n\twhich means to NOT include those files.\n");
                     goto prompt;
                  }
               }
@@ -703,13 +715,13 @@ sub start_ssh {
   push @ssh_command, "-S", $socket;
   my @start_ssh_command = @ssh_command;
   push @start_ssh_command, "-o", "ControlMaster=yes", "-f", "-N";
-  print "Starting ssh connection..\n";
+  print _g("Starting ssh connection..\n");
   system(@start_ssh_command);
   my $ret = $? >> 8;
   @end_ssh_command = @ssh_command;
   push @end_ssh_command, "-O", "exit";
   if($ret != 0) {
-    exwerror("Failed to make the ssh connection, ssh's return code was: " . $ret);
+    exwerror(_g("Failed to make the ssh connection, ssh's return code was: ") . $ret);
   }
   my @svn_ssh = qw/ssh/;
   push @svn_ssh, "-S", $socket;
@@ -726,6 +738,10 @@ sub END {
   stop_ssh();
 }
 
+sub _g {
+	return gettext(shift);
+}
+
 1;
 ##
 ##### /common ###
